export function getCookie(name, cookies) {
    if (cookies == null) {
        if (typeof window === 'undefined') {
            return undefined;
        }
        cookies = document.cookie;
    }
    const kv = cookies && cookies.split(';').find((part) => part.trim().startsWith(name));
    if (!kv)
        return undefined;
    const cookieValue = kv.split('=')[1];
    if (!cookieValue)
        return undefined;
    return decodeURIComponent(cookieValue.trim());
}
export function setCookie(name, value, options = {}) {
    document.cookie = formatCookie(name, value, options);
}
function formatCookie(name, value, options = {}) {
    if (options.expires instanceof Date)
        options.expires = options.expires.toUTCString();
    const updatedCookie = {
        [encodeURIComponent(name)]: encodeURIComponent(value),
        sameSite: 'strict',
        path: '/',
        ...options,
    };
    const cookie = Object.entries(updatedCookie)
        .filter(([key]) => key !== 'secure')
        .map((kv) => kv.join('='))
        .join(';');
    return options.secure === false ? cookie : `${cookie};secure`;
}
